"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const Hapi = require("hapi");
const injector_class_1 = require("./injector.class");
const interfaces_1 = require("./interfaces");
const _ = require("lodash");
const Boom = require("boom");
const DEFAULT_APP_CONFIG = {
    port: 3000
};
const DEFAULT_BOOTSTRAP_OPTIONS = {
    injector: new injector_class_1.Injector()
};
function bootstrapWithOptions(BootstrapedApps, options = DEFAULT_BOOTSTRAP_OPTIONS) {
    const apps = [];
    for (const BootstrapedApp of BootstrapedApps) {
        const server = new Hapi.Server();
        const app = new BootstrapedApp(server);
        server.connection(_.extend(Reflect.getMetadata('hapiour:config', BootstrapedApp), DEFAULT_APP_CONFIG));
        _.invoke(app, 'onInit');
        const plugins = getPluginsRecurs(options.injector, Reflect.getMetadata('hapiour:plugins', BootstrapedApp));
        const routes = getRoutesWithConfigRecurs(options.injector, BootstrapedApp, server);
        if (_.isEmpty(plugins)) {
            _.invoke(app, 'onRegister');
            server.route(routes);
            server.start((err) => {
                if (err) {
                    throw new Error(err);
                }
                _.invoke(app, 'onStart');
            });
        }
        else {
            server.register(plugins, (registerErr) => {
                if (registerErr) {
                    throw new Error(registerErr);
                }
                _.invoke(app, 'onRegister');
                server.route(routes);
                server.start((startErr) => {
                    if (startErr) {
                        throw new Error(startErr);
                    }
                    _.invoke(app, 'onStart');
                });
            });
        }
        apps.push(app);
    }
    return apps;
}
exports.bootstrapWithOptions = bootstrapWithOptions;
function getPluginsRecurs(injector, Plugins) {
    let plugins = [];
    if (_.isArray(Plugins)) {
        for (const Plugin of Plugins) {
            if (Plugin instanceof Array) {
                plugins = _.concat(plugins, getPluginsRecurs(injector, Plugin));
            }
            else if (Plugin['register']) {
                plugins.push(Plugin);
            }
            else {
                const plugin = injector.get(Plugin);
                if (Reflect.hasMetadata('hapiour:register', Plugin)) {
                    plugin.register = Reflect.getMetadata('hapiour:register', Plugin);
                }
                else if (Reflect.hasMetadata('hapiour:attributes', Plugin)) {
                    plugin.register = plugin.register.bind(plugin);
                    plugin.register.attributes = Reflect.getMetadata('hapiour:attributes', Plugin);
                }
                plugins.push(plugin);
            }
        }
    }
    return plugins;
}
function getMappingArgs(request, reply, target) {
    let args = [];
    if (Reflect.hasMetadata('hapiour:mappings', target)) {
        let mapping_types = Reflect.getOwnMetadata("hapiour:mappings", target);
        if (request.params) {
            mapping_types.forEach((mappingType) => {
                switch (mappingType.type) {
                    case interfaces_1.MType.req_attr:
                        args.unshift(request[mappingType.param_name]);
                        break;
                    case interfaces_1.MType.require_req_attr:
                        let value = request[mappingType.param_name];
                        if (!value) {
                            throw Boom.badData(`${mappingType.param_name} is required`);
                        }
                        args.unshift(value);
                        break;
                    case interfaces_1.MType.reply:
                        args.unshift(reply);
                        break;
                    case interfaces_1.MType.request:
                        args.unshift(request);
                        break;
                    default:
                        let val = request.params[mappingType.param_name];
                        if (mappingType.parse_it instanceof Function) {
                            args.unshift(mappingType.parse_it(val));
                        }
                        else {
                            args.unshift(val);
                        }
                }
            });
        }
    }
    return args;
}
function overrideModuleHandler(injector, server, mod, route, modConfig) {
    let rHandler = route.handler;
    let config = route.config || {};
    config.pre = config.pre || [];
    if (modConfig.interceptors) {
        modConfig.interceptors.forEach((it, index) => {
            const interceptor = injector.get(it);
            config.pre.unshift({
                method: (request, reply) => {
                    let fn = interceptor.intercept;
                    let args = getMappingArgs(request, reply, fn);
                    fn.apply(interceptor, args)
                        .then(() => reply.continue())
                        .catch((e) => reply(e));
                },
            });
        });
    }
    route.config = config;
    route.handler = (request, reply) => {
        try {
            let args = getMappingArgs(request, reply, rHandler);
            rHandler.apply(mod, args)
                .then((rsp) => reply(rsp))
                .catch((e) => reply.response(e));
        }
        catch (e) {
            reply.response(e);
        }
    };
}
function getRoutesWithConfigRecurs(injector, item, server, parent) {
    let routes = [];
    if (Reflect.hasMetadata('hapiour:modules', item)) {
        const modules = Reflect.getMetadata('hapiour:modules', item);
        const parentConfig = (parent) ? Reflect.getMetadata('hapiour:config', parent) : {};
        for (const Mod of modules) {
            const config = Reflect.getMetadata('hapiour:config', Mod);
            let modRoutes = _.cloneDeep(Reflect.getMetadata('hapiour:routes', Mod));
            const mod = injector.get(Mod);
            modRoutes = _.each(modRoutes, (route) => {
                route.path = _.get(parentConfig, 'basePath', '') + config.basePath + route.path;
                overrideModuleHandler(injector, server, mod, route, config);
            });
            routes = _.concat(routes, modRoutes, getRoutesWithConfigRecurs(injector, Mod, server, item));
        }
    }
    return routes;
}
function bootstrap(...BootstrapedApps) {
    return bootstrapWithOptions(BootstrapedApps);
}
exports.bootstrap = bootstrap;
//# sourceMappingURL=bootstrap.js.map